<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xep="http://www.renderx.com/XEP/xep" 
    version="1.0">
    
    <xsl:key name="xep:page" match="xep:page" use="xep:text/@xpath"/>
    <xsl:variable name="wrapper_xpath"><![CDATA[/*[1][name()='wrapper']]]></xsl:variable>

    <xsl:template match="@* | node()">
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="node()"/>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template match="xep:text">
        <xsl:variable name="page-number" select="ancestor::xep:page/@page-number"/>
        <xsl:variable name="xpath" select="@xpath"/>
        <xsl:variable name="gen-page-id" select="generate-id(ancestor::xep:page)"/>
        <xsl:choose>
            <!-- mark all content in region-before or region-after as Artifact -->
            <xsl:when test="contains($xpath,'static-content')">
                <xsl:element name="xep:text">
                    <xsl:apply-templates select="@*[name()!='xpath']"/>
                    <xsl:variable name="static-content">
                        <xsl:text>[name()=&apos;static-content&apos;]</xsl:text>
                    </xsl:variable>
                    <xsl:attribute name="xpath">
                        <xsl:value-of select="concat(substring-before($xpath,$static-content),$static-content,$wrapper_xpath)"/>
                    </xsl:attribute>
                    <!-- Force mark as an Artifact -->
                    <xsl:attribute name="pdf-structure-tag">
                        <xsl:text>Artifact</xsl:text>
                    </xsl:attribute>
                    <!-- Force mark type Pagination -->
                    <xsl:attribute name="pdf-artifact-type">
                        <xsl:text>Pagination</xsl:text>
                    </xsl:attribute>
                </xsl:element>
            </xsl:when>
            <!-- fix table repeating headers -->
            <xsl:when test="not($gen-page-id = generate-id(key('xep:page',$xpath)[1])) and contains($xpath,'table-header')">
                <xsl:comment>fix repeating table headers</xsl:comment>
                <xsl:element name="xep:text">
                    <xsl:apply-templates select="@*[name()!='xpath']"/>
                    <xsl:attribute name="xpath">
                        <xsl:value-of select="concat(@xpath, $wrapper_xpath)"/>
                    </xsl:attribute>
                    <xsl:attribute name="pdf-structure-tag">Artifact</xsl:attribute>
                </xsl:element>
            </xsl:when>
            <!-- fix text boundaries -->
            <xsl:when test="not(normalize-space(@value) = '') and not(substring(@value, string-length(@value) - 1,1)=' ') and following::xep:text[1][@xpath=$xpath][not(starts-with(@value, ' '))]">
                <xsl:comment>fix text boundaries</xsl:comment>
                <xsl:element name="xep:text" namespace="http://www.renderx.co/XEP/xep">
                    <xsl:attribute name="value">
                        <xsl:value-of select="concat(@value, ' ')"/>
                    </xsl:attribute>
                    <xsl:apply-templates select="@*[not(name()='value')]"/>
                </xsl:element>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy>
                    <xsl:apply-templates select="@*"/>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>
